@@ -2,31 +2,35 @@
 Cell Type Model
 """
 
-from .abstract import AstirModel
-from astir.data import SCDataset
-from .celltype_recognet import TypeRecognitionNet
-import torch
-import seaborn as sns
 import re
-from typing import Tuple, List, Dict, Optional, Generator, Union
 import warnings
-from tqdm import trange
+from collections import OrderedDict
+from typing import Dict, Generator, List, Optional, Tuple, Union
+
+import h5py
+import numpy as np
+import pandas as pd
+import seaborn as sns
+import torch
+import torch.nn as nn
+import torch.nn.functional as F
+from torch.distributions import Dirichlet
+from scipy import stats
+from sklearn.preprocessing import StandardScaler
 from torch.autograd import Variable
 from torch.distributions import (
+    LowRankMultivariateNormal,
+    MultivariateNormal,
     Normal,
     StudentT,
-    MultivariateNormal,
-    LowRankMultivariateNormal,
 )
-import torch.nn.functional as F
-import torch.nn as nn
-from torch.utils.data import Dataset, DataLoader
-import pandas as pd
-import numpy as np
-from sklearn.preprocessing import StandardScaler
-from scipy import stats
-from collections import OrderedDict
-import h5py
+from torch.utils.data import DataLoader, Dataset
+from tqdm import trange
+
+from astir.data import SCDataset
+
+from .abstract import AstirModel
+from .celltype_recognet import TypeRecognitionNet
 
 
 class CellTypeModel(AstirModel):
@@ -51,8 +55,7 @@ class CellTypeModel(AstirModel):
             self._param_init()
 
     def _param_init(self) -> None:
-        """ Initializes parameters and design matrices.
-        """
+        """Initializes parameters and design matrices."""
         if self._dset is None:
             raise Exception("the dataset is not provided")
         G = self._dset.get_n_features()
@@ -64,24 +67,35 @@ class CellTypeModel(AstirModel):
 
         # Establish data
         self._data: Dict[str, torch.Tensor] = {
-            "log_alpha": torch.log(torch.ones(C + 1, dtype=self._dtype) / (C + 1)).to(
-                self._device
-            ),
+            # "log_alpha": torch.log(torch.ones(C + 1, dtype=self._dtype) / (C + 1)).to(
+            #     self._device
+            # ),
             "rho": self._dset.get_marker_mat().to(self._device),
         }
+
+        self._alpha_prior = Dirichlet(
+            torch.ones(C + 1, dtype=self._dtype).to(self._device) * (C + 1)
+        )
+
         # Initialize mu, log_delta
         delta_init_mean = torch.log(
             torch.log(torch.tensor(3.0, dtype=self._dtype))
         )  # the log of the log of this is the multiplier
         t = torch.distributions.Normal(
-            delta_init_mean.clone().detach().to(self._dtype),
+            # delta_init_mean.clone().detach().to(self._dtype),
+            torch.tensor(0, dtype=self._dtype),
             torch.tensor(0.1, dtype=self._dtype),
         )
         log_delta_init = t.sample((G, C + 1))
-        mu_init = torch.log(self._dset.get_mu()).to(self._device)
-        mu_init = mu_init - (
-            self._data["rho"] * torch.exp(log_delta_init).to(self._device)
-        ).mean(1)
+
+        mu_init = torch.log(
+            torch.tensor(self._dset.get_mu_init(), dtype=self._dtype)
+        ).to(self._device)
+        # mu_init = torch.log(self._dset.get_mu()).to(self._device)
+        # mu_init = mu_init - (
+        #     self._data["rho"] * torch.exp(log_delta_init).to(self._device)
+        # ).mean(1)
+
         mu_init = mu_init.reshape(-1, 1)
 
         # Create initialization dictionary
@@ -90,6 +104,7 @@ class CellTypeModel(AstirModel):
             "log_sigma": torch.log(self._dset.get_sigma()).to(self._device),
             "log_delta": log_delta_init,
             "p": torch.zeros((G, C + 1), dtype=self._dtype, device=self._device),
+            "alpha_logits": torch.ones(C + 1, dtype=self._dtype, device=self._device),
         }
         P = self._dset.get_design().shape[1]
         # Add additional columns of mu for anything in the design matrix
@@ -107,7 +122,7 @@ class CellTypeModel(AstirModel):
             self._variables[n].requires_grad = True
 
     def load_hdf5(self, hdf5_name: str) -> None:
-        """ Initializes Cell Type Model from a hdf5 file type
+        """Initializes Cell Type Model from a hdf5 file type
 
         :param hdf5_name: file path
         """
@@ -120,9 +135,9 @@ class CellTypeModel(AstirModel):
                 "log_sigma": torch.tensor(np.array(param["log_sigma"])),
                 "log_delta": torch.tensor(np.array(param["log_delta"])),
                 "p": torch.tensor(np.array(param["p"])),
+                "alpha_logits": torch.tensor(np.array(param["alpha_logits"])),
             }
             self._data = {
-                "log_alpha": torch.tensor(np.array(param["log_alpha"])),
                 "rho": torch.tensor(np.array(param["rho"])),
             }
             self._losses = torch.tensor(np.array(grp["losses"]["losses"]))
@@ -146,7 +161,7 @@ class CellTypeModel(AstirModel):
     def _forward(
         self, Y: torch.Tensor, X: torch.Tensor, design: torch.Tensor
     ) -> torch.Tensor:
-        """ One forward pass.
+        """One forward pass.
 
         :param Y: a sample from the dataset
         :param X: normalized sample data
@@ -159,12 +174,12 @@ class CellTypeModel(AstirModel):
         C = self._dset.get_n_classes()
         N = Y.shape[0]
 
-        Y_spread = Y.reshape(-1, G, 1).repeat(1, 1, C + 1)
+        Y_spread = Y.view(N, 1, G).repeat(1, C + 1, 1)
 
-        delta_tilde = torch.exp(self._variables["log_delta"])  # + np.log(0.5)
+        delta_tilde = torch.exp(self._variables["log_delta"])
         mean = delta_tilde * self._data["rho"]
         mean2 = torch.mm(design, self._variables["mu"].T)  ## N x P * P x G
-        mean2 = mean2.reshape(-1, G, 1).repeat(1, 1, C + 1)
+        mean2 = mean2.view(-1, G, 1).repeat(1, 1, C + 1)
         mean = mean + mean2
 
         # now do the variance modelling
@@ -174,19 +189,21 @@ class CellTypeModel(AstirModel):
         v1 = (self._data["rho"] * p).T * sigma
         v2 = torch.pow(sigma, 2) * (1 - torch.pow(self._data["rho"] * p, 2)).T
 
-        v1 = v1.reshape(1, C + 1, G, 1).repeat(N, 1, 1, 1)  # extra 1 is the "rank"
-        v2 = v2.reshape(1, C + 1, G).repeat(N, 1, 1) + 1e-6
+        v1 = v1.view(1, C + 1, G, 1).repeat(N, 1, 1, 1)  # extra 1 is the "rank"
+        v2 = v2.view(1, C + 1, G).repeat(N, 1, 1) + 1e-6
 
         dist = LowRankMultivariateNormal(
             loc=torch.exp(mean).permute(0, 2, 1), cov_factor=v1, cov_diag=v2
         )
 
-        log_p_y_on_c = dist.log_prob(Y_spread.permute(0, 2, 1))
+        log_p_y_on_c = dist.log_prob(Y_spread)
 
-        gamma = self._recog.forward(X)
-        elbo = (
-            gamma * (log_p_y_on_c + self._data["log_alpha"] - torch.log(gamma))
-        ).sum()
+        gamma, log_gamma = self._recog.forward(X)
+        log_alpha = F.log_softmax(self._variables["alpha_logits"], dim=0)
+        alpha = F.softmax(self._variables["alpha_logits"], dim=0)
+        mix_prior = self._alpha_prior.log_prob(alpha)
+
+        elbo = (gamma * (log_p_y_on_c + log_alpha - log_gamma)).sum() + mix_prior
 
         return -elbo
 
@@ -199,7 +216,7 @@ class CellTypeModel(AstirModel):
         delta_loss_batch: int = 10,
         msg: str = "",
     ) -> None:
-        """ Runs train loops until the convergence reaches delta_loss for
+        """Runs train loops until the convergence reaches delta_loss for
         delta_loss_batch sizes or for max_epochs number of times
 
         :param max_epochs: number of train loop iterations, defaults to 50
@@ -259,7 +276,7 @@ class CellTypeModel(AstirModel):
 
         # Save output
         self._assignment = pd.DataFrame(
-            self._recog.forward(exprs_X).detach().cpu().numpy()
+            self._recog.forward(exprs_X)[0].detach().cpu().numpy()
         )
         self._assignment.columns = self._dset.get_classes() + ["Other"]
         self._assignment.index = self._dset.get_cell_names()
@@ -278,55 +295,88 @@ class CellTypeModel(AstirModel):
         :return: the resulting cell type assignment
         """
         _, exprs_X, _ = new_dset[:]
-        g = pd.DataFrame(self._recog.forward(exprs_X).detach().cpu().numpy())
+        g = pd.DataFrame(self._recog.forward(exprs_X)[0].detach().cpu().numpy())
         return g
 
     def get_recognet(self) -> TypeRecognitionNet:
-        """ Getter for the recognition net.
+        """Getter for the recognition net.
 
         :return: the trained recognition net
         """
         return self._recog
 
     def _most_likely_celltype(
-        self, row: pd.DataFrame, threshold: float, cell_types: List[str]
+        self,
+        row: pd.DataFrame,
+        threshold: float,
+        cell_types: List[str],
+        assignment_type: str,
     ) -> str:
         """Given a row of the assignment matrix, return the most likely cell type
 
         :param row: the row of cell assignment matrix to be evaluated
         :param threshold: the higher bound of the maximun probability to classify a cell as `Unknown`
         :param cell_types: the names of cell types, in the same order as the features of the row
+        :param assignment_type: See
+        :meth:`astir.CellTypeModel.get_celltypes` for full documentation
         :return: the most likely cell type of this cell
         """
         row = row.values
         max_prob = np.max(row)
 
-        if max_prob < threshold:
-            return "Unknown"
+        if assignment_type == "threshold":
+            if max_prob < threshold:
+                return "Unknown"
+        elif assignment_type == "max":
+            if sum(row == max_prob) > 1:
+                return "Unknown"
 
         return cell_types[np.argmax(row)]
 
     def get_celltypes(
-        self, threshold: float = 0.7, prob_assign: Optional[pd.DataFrame] = None
+        self,
+        threshold: float = 0.7,
+        assignment_type: str = "threshold",
+        prob_assign: Optional[pd.DataFrame] = None,
     ) -> pd.DataFrame:
-        """ Get the most likely cell types
-
-        A cell is assigned to a cell type if the probability is greater than threshold.
-        If no cell types have a probability higher than threshold, then "Unknown" is returned.
+        """Get the most likely cell types. A cell is assigned to a cell type
+        if the probability is greater than threshold.
+        If no cell types have a probability higher than threshold,
+        then "Unknown" is returned.
 
+        :param assignment_type: either 'threshold' or 'max'. If threshold,
+            type assignment is based on whether the probability threshold is
+            above prob_assignment. If 'max', type assignment is based on the max
+            probability value or "unknown" if there are multiple max
+            probabilities. Defaults to 'threshold'.
         :param threshold: the probability threshold above which a cell is
             assigned to a cell type, defaults to 0.7
-        :return: a data frame with most likely cell types for each 
+        :return: a data frame with most likely cell types for each
         """
         if prob_assign is None:
             type_probability = self.get_assignment()
         else:
             type_probability = prob_assign
+
+        if assignment_type != "threshold" and assignment_type != "max":
+            warnings.warn(
+                "Wrong assignment type. Defaults the assignment " "type to threshold."
+            )
+            assignment_type = "threshold"
+
+        if assignment_type == "max" and prob_assign is not None:
+            warnings.warn(
+                "Assignment type is 'max' but probability "
+                "threshold value was passed in. Probability "
+                "threshold value will be ignored."
+            )
+
         cell_types = list(type_probability.columns)
 
         cell_type_assignments = type_probability.apply(
             self._most_likely_celltype,
             axis=1,
+            assignment_type=assignment_type,
             threshold=threshold,
             cell_types=cell_types,
         )
